@@ -126,6 +126,7 @@ public:
     bool pageChangeSupressed = false;
     bool mSaveStatistics = true;
     bool mDefaultsIndicatorsVisible = false;
+    KCModule::Buttons mButtonMask = ~KCModule::Buttons(KCModule::NoAdditionalButton);
 };
 
 ModuleView::ModuleView(QWidget *parent)
@@ -248,6 +249,8 @@ void ModuleView::addModule(MenuItem *item, const QStringList &args)
     if (item->isExternalAppModule()) {
         auto *externalWidget = new ExternalAppModule(this, KService::Ptr(new KService(item->metaData().metaDataFileName())));
         moduleScroll->setWidget(externalWidget);
+        d->mCustomHeader->setText(item->metaData().name()); // We have to set this manually, BUG: 448672
+        page->setName(QString());
     } else { // It must be a normal module then
         auto *moduleProxy = new KCModuleProxy(data, moduleScroll, args);
         moduleScroll->setWidget(moduleProxy);
@@ -257,12 +260,12 @@ void ModuleView::addModule(MenuItem *item, const QStringList &args)
     }
 
     d->mPagesPluginIdMap.insert(page, data.name());
-    updatePageIconHeader(page, true);
+    updatePageIconHeader(page);
     // Add the new page
     d->mPageWidget->addPage(page);
 }
 
-void ModuleView::updatePageIconHeader(KPageWidgetItem *page, bool light)
+void ModuleView::updatePageIconHeader(KPageWidgetItem *page)
 {
     if (!page) {
         // Page is invalid. Probably means we have a race condition during closure of everyone so do nothing
@@ -270,24 +273,29 @@ void ModuleView::updatePageIconHeader(KPageWidgetItem *page, bool light)
     }
 
     KCModuleProxy *moduleProxy = d->mPages.value(page);
-
     if (!moduleProxy || !moduleProxy->metaData().isValid()) {
         // Seems like we have some form of a race condition going on here...
         return;
     }
 
-    const QString currentModuleName = moduleProxy->metaData().name();
-    page->setHeader(currentModuleName);
+    const QString moduleName = moduleProxy->metaData().name();
+    page->setHeader(moduleName);
+    d->mCustomHeader->setText(moduleName);
     page->setIcon(QIcon::fromTheme(moduleProxy->metaData().iconName()));
 
-    const bool isQml = (moduleProxy->realModule() && moduleProxy->realModule()->inherits("KCModuleQml"));
-    if (d->mPageWidget->currentPage() && d->mPageWidget->currentPage()->name() == currentModuleName) {
-        d->mCustomHeader->setVisible(!isQml);
-    }
-    page->setHeaderVisible(false);
-
-    if (light) {
-        return;
+    const bool isQml = moduleProxy->realModule() && moduleProxy->realModule()->inherits("KCModuleQml");
+    if (isQml) {
+        // QML KCM: We don't use any widgets header
+        d->mCustomHeader->setVisible(false);
+        page->setHeaderVisible(false);
+    } else if (faceType() == KPageView::Plain) {
+        // QWidgets KCM on Sidebar mode: Use the custom header
+        d->mCustomHeader->setVisible(true);
+        page->setHeaderVisible(false);
+    } else {
+        // QWidgets KCM on Icons mode: Use the module's header
+        d->mCustomHeader->setVisible(false);
+        page->setHeaderVisible(true);
     }
 }
 
@@ -401,71 +409,66 @@ void ModuleView::activeModuleChanged(KPageWidgetItem *current, KPageWidgetItem *
     if (d->pageChangeSupressed) {
         return;
     }
+
     // We need to get the state of the now active module
     stateChanged();
 
     KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
-    if (activeModule) {
-        // TODO: if we'll ever need statistics for kinfocenter modules, save them with an URL like "kinfo:"
-
-        if (d->mSaveStatistics && activeModule->metaData().pluginId() != QStringLiteral("kcm_landingpage")) {
-            KActivities::ResourceInstance::notifyAccessed(QUrl(QStringLiteral("kcm:") + activeModule->metaData().pluginId()),
-                                                          QStringLiteral("org.kde.systemsettings"));
-        }
-
-        d->mCustomHeader->setText(activeModule->metaData().name());
-
-        d->mLayout->setContentsMargins(0, 0, 0, 0);
-        d->mLayout->setSpacing(0);
-        d->mButtons->setContentsMargins(style()->pixelMetric(QStyle::PM_LayoutLeftMargin),
-                                        0, // Remove extra space between KCM content and bottom buttons
-                                        style()->pixelMetric(QStyle::PM_LayoutRightMargin),
-                                        style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
-
-        const bool isQml = (activeModule && activeModule->realModule() && activeModule->realModule()->inherits("KCModuleQml"));
-        d->mCustomHeader->setVisible(!isQml);
-        current->setHeaderVisible(!isQml);
-
-        QGridLayout *gridLayout = static_cast<QGridLayout *>(d->mPageWidget->layout());
-
-        // QML KCM
-        if (isQml) {
-            gridLayout->setHorizontalSpacing(0);
-            d->mCustomHeader->setVisible(false);
-            current->setHeaderVisible(false);
-
-            // QWidget KCM
-        } else {
-            // Sidebar mode
-            if (faceType() == KPageView::Plain) {
-                d->mCustomHeader->setVisible(true);
-                current->setHeaderVisible(false);
-                gridLayout->setHorizontalSpacing(0);
-
-                // Icons mode
-            } else {
-                d->mCustomHeader->setVisible(false);
-                current->setHeaderVisible(true);
-                gridLayout->setHorizontalSpacing(style()->pixelMetric(QStyle::PM_LayoutHorizontalSpacing));
-            }
-        }
-
-        moduleShowDefaultsIndicators(d->mDefaultsIndicatorsVisible);
+    if (!activeModule) {
+        return;
     }
+
+    // TODO: if we'll ever need statistics for kinfocenter modules, save them with an URL like "kinfo:"
+    if (d->mSaveStatistics && activeModule->metaData().pluginId() != QStringLiteral("kcm_landingpage")) {
+        KActivities::ResourceInstance::notifyAccessed(QUrl(QStringLiteral("kcm:") + activeModule->metaData().pluginId()),
+                                                      QStringLiteral("org.kde.systemsettings"));
+    }
+
+    d->mLayout->setContentsMargins(0, 0, 0, 0);
+    d->mLayout->setSpacing(0);
+    d->mButtons->setContentsMargins(style()->pixelMetric(QStyle::PM_LayoutLeftMargin),
+                                    0, // Remove extra space between KCM content and bottom buttons
+                                    style()->pixelMetric(QStyle::PM_LayoutRightMargin),
+                                    style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
+    d->mPageWidget->layout()->setSpacing(0);
+    if (auto titleWidget = qobject_cast<KTitleWidget *>(d->mPageWidget->pageHeader())) {
+        titleWidget->layout()->setContentsMargins(style()->pixelMetric(QStyle::PM_LayoutLeftMargin),
+                                                  style()->pixelMetric(QStyle::PM_LayoutRightMargin),
+                                                  style()->pixelMetric(QStyle::PM_LayoutRightMargin),
+                                                  style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
+    }
+
+    updatePageIconHeader(current);
+    moduleShowDefaultsIndicators(d->mDefaultsIndicatorsVisible);
 }
 
 void ModuleView::stateChanged()
+{
+    updatePageIconHeader(d->mPageWidget->currentPage());
+    updateButtons();
+
+    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    Q_EMIT moduleChanged(activeModule && activeModule->isChanged());
+}
+
+void ModuleView::updateButtons()
 {
     KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     KAuth::Action moduleAction;
     bool change = false;
     bool defaulted = false;
     KCModule::Buttons buttons = KCModule::NoAdditionalButton;
+
     if (activeModule) {
-        buttons = activeModule->buttons();
+        buttons = activeModule->buttons() & d->mButtonMask;
         change = activeModule->isChanged();
         defaulted = activeModule->defaulted();
 
+        // Do not display Help button if there is no docPath available
+        if (activeModule->metaData().value(QStringLiteral("X-DocPath")).isEmpty()) {
+            buttons &= ~KCModule::Help;
+        }
+
         disconnect(d->mApplyAuthorize, SIGNAL(authorized(KAuth::Action)), this, SLOT(moduleSave()));
         disconnect(d->mApply, SIGNAL(clicked()), this, SLOT(moduleSave()));
         if (activeModule->realModule()->authAction().isValid()) {
@@ -476,11 +479,6 @@ void ModuleView::stateChanged()
         }
     }
 
-    updatePageIconHeader(d->mPageWidget->currentPage());
-
-    KCModuleProxy *moduleProxy = d->mPages.value(d->mPageWidget->currentPage());
-    d->mCustomHeader->setVisible(!moduleProxy || !moduleProxy->realModule()->inherits("KCModuleQml"));
-
     d->mApplyAuthorize->setAuthAction(moduleAction);
     d->mDefault->setEnabled(!defaulted);
     d->mDefault->setVisible(buttons & KCModule::Default);
@@ -490,7 +488,8 @@ void ModuleView::stateChanged()
     d->mReset->setVisible(buttons & KCModule::Apply);
     d->mHelp->setEnabled(buttons & KCModule::Help);
     d->mHelp->setVisible(buttons & KCModule::Help);
-    Q_EMIT moduleChanged(change);
+
+    d->mButtons->setVisible(buttons != KCModule::NoAdditionalButton);
 }
 
 void ModuleView::keyPressEvent(QKeyEvent *event)
@@ -534,7 +533,8 @@ bool ModuleView::saveStatistics() const
 
 void ModuleView::setApplyVisible(bool visible)
 {
-    d->mApply->setVisible(visible);
+    d->mButtonMask.setFlag(KCModule::Apply, visible);
+    updateButtons();
 }
 
 bool ModuleView::isApplyVisible() const
@@ -544,7 +544,8 @@ bool ModuleView::isApplyVisible() const
 
 void ModuleView::setDefaultsVisible(bool visible)
 {
-    d->mDefault->setVisible(visible);
+    d->mButtonMask.setFlag(KCModule::Default, visible);
+    updateButtons();
 }
 
 bool ModuleView::isDefaultsVisible() const
@@ -552,11 +553,6 @@ bool ModuleView::isDefaultsVisible() const
     return d->mDefault->isVisible();
 }
 
-void ModuleView::setResetVisible(bool visible)
-{
-    d->mReset->setVisible(visible);
-}
-
 bool ModuleView::isResetVisible() const
 {
     return d->mReset->isVisible();
